package es.imunnic.inversionitasBot.services;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import es.imunnic.inversionitasBot.TelegramBot;
import org.json.JSONArray;
import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

@Component
public class ApiService {
  //@Value("${apiservice.url}")
  private static String API_BASE_URL="http://fastapi-app:8000/news";

  private static final Logger logger = LoggerFactory.getLogger(ApiService.class);

  // Método genérico para hacer solicitudes GET
  public static String sendGetRequest(String endpoint) {
    try {
      URL url = new URL(API_BASE_URL + endpoint);
      HttpURLConnection connection = (HttpURLConnection) url.openConnection();
      connection.setRequestMethod("GET");

      BufferedReader in = new BufferedReader(new InputStreamReader(connection.getInputStream()));
      StringBuilder content = new StringBuilder();
      String inputLine;
      while ((inputLine = in.readLine()) != null) {
        content.append(inputLine);
      }
      in.close();
      connection.disconnect();
      logger.info("Mensaje recibido: {}", content.toString());
      return content.toString();
    } catch (Exception e) {
      return "{\"error\": \"Error al obtener datos de la API\"}";
    }
  }

  // 1️⃣ Obtener el conteo de noticias por fuente
  public static String getNewsCountBySource() {
    return sendGetRequest("/count/by-source");
  }

  // 2️⃣ Obtener el conteo de noticias por autor
  public static String getNewsCountByAuthor() {
    return sendGetRequest("/count/by-author");
  }

  // 3️⃣ Obtener noticia por ID
  public static String getNewsById(int newsId) {
    return sendGetRequest("/" + newsId);
  }

  // 4️⃣ Obtener el conteo de noticias por keyword
  public static String getNewsCountByKeyword(String fechaInicio, String fechaFin) {
    String params = "?fechaInicio=" + fechaInicio + "&fechaFin=" + fechaFin;
    return sendGetRequest("/count/by-keyword" + params);
  }

  // 5️⃣ Obtener todas las keywords
  public static List<String> getAllKeywords() {
    String jsonResponse = sendGetRequest("/keywords/");

    // Crear una lista para almacenar las keywords
    List<String> keywordsList = new ArrayList<>();

    try {
      // Convertir la respuesta JSON en un objeto JSONObject
      JSONObject jsonObject = new JSONObject(jsonResponse);

      // Obtener el array de keywords
      JSONArray keywordsArray = jsonObject.getJSONArray("keywords");

      // Recorrer el array y agregar cada keyword a la lista
      for (int i = 0; i < keywordsArray.length(); i++) {
        keywordsList.add(keywordsArray.getString(i));
      }
    } catch (Exception e) {
      // Manejo de excepciones, puedes hacer más aquí si es necesario
      logger.error("Error al procesar los datos JSON", e);
    }

    // Retornar la lista de keywords
    return keywordsList;
  }

  // 6️⃣ Contar noticias por fuente con una keyword y rango de fechas
  public static String getNewsCountBySourceForKeyword(String keyword, String fechaInicio, String fechaFin) {
    String params = "?keyword=" + encodeParam(keyword) + "&fechaInicio=" + fechaInicio + "&fechaFin=" + fechaFin;
    return sendGetRequest("/count/by-source/keyword/date-range" + params);
  }

  // 7️⃣ Obtener títulos de noticias por keyword en un rango de fechas
  public static List<String> getTitlesByKeyword(String keyword, String fechaInicio, String fechaFin) {
    String params = "?keyword=" + encodeParam(keyword) + "&fecha_inicio=" + fechaInicio + "&fecha_fin=" + fechaFin;
    String jsonResponse = sendGetRequest("/titles/by-keyword/date-range" + params);

    List<String> titlesList = new ArrayList<>();

    try {
      // Convertir la respuesta JSON en un objeto JSONObject
      JSONObject jsonObject = new JSONObject(jsonResponse);

      // Verificar si el JSON contiene el array de títulos
      if (jsonObject.has("titles")) {
        // Obtener el array de títulos
        JSONArray titlesArray = jsonObject.getJSONArray("titles");

        // Recorrer el array y agregar cada título a la lista
        for (int i = 0; i < titlesArray.length(); i++) {
          titlesList.add(titlesArray.getString(i));
        }
      }
    } catch (Exception e) {
      // Manejo de excepciones, puedes hacer más aquí si es necesario
      logger.error("Error al procesar los títulos de noticias", e);
    }

    // Retornar la lista de títulos
    return titlesList;
  }

  // 8️⃣ Obtener todos los títulos de noticias en un rango de fechas
  public static String getNewsTitles(String fechaInicio, String fechaFin) {
    String params = "?fechaInicio=" + fechaInicio + "&fechaFin=" + fechaFin;
    return sendGetRequest("/titles/" + params);
  }

  // 9️⃣ Obtener todas las noticias con paginación
  public static String getAllNews(String fechaInicio, String fechaFin, int page, int pageSize) {
    String params = "?fechaInicio=" + fechaInicio + "&fechaFin=" + fechaFin +
        "&page=" + page + "&pageSize=" + pageSize;
    return sendGetRequest("/" + params);
  }

  // 🔟 Método para formatear LocalDateTime a String (si se usa en Java)
  public static String formatDate(LocalDateTime dateTime) {
    DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd'T'HH:mm:ss");
    return dateTime.format(formatter);
  }

  // 🌎 Método auxiliar para codificar parámetros en URL
  private static String encodeParam(String param) {
    return URLEncoder.encode(param, StandardCharsets.UTF_8);
  }
}
