package es.imunnic.inversionitasBot.services;

import es.imunnic.inversionitasBot.NewsArticle;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.jsoup.select.Elements;
import com.rometools.rome.feed.synd.SyndEntry;
import com.rometools.rome.feed.synd.SyndFeed;
import com.rometools.rome.io.SyndFeedInput;
import com.rometools.rome.io.XmlReader;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import java.net.HttpURLConnection;
import java.net.URL;
import java.time.LocalDate;
import java.time.ZoneId;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Service
public class GoogleNewsScraperService {

  private static final Logger logger = LoggerFactory.getLogger(GoogleNewsScraperService.class);
  private static final String USER_AGENT = "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.124 Safari/537.36";

  public List<NewsArticle> searchNews(String query) {
    List<NewsArticle> newsList = new ArrayList<>();
    String rssUrl = "https://news.google.com/rss/search?q=" + query.replace(" ", "+") + "&hl=es&gl=ES&ceid=ES%3Aes";

    logger.info("Buscando noticias para: {}", query);
    logger.info("URL del RSS: {}", rssUrl);

    try {
      URL url = new URL(rssUrl);
      HttpURLConnection conn = (HttpURLConnection) url.openConnection();
      conn.setRequestProperty("User-Agent", USER_AGENT);

      SyndFeedInput input = new SyndFeedInput();
      SyndFeed feed = input.build(new XmlReader(conn.getInputStream()));

      List<SyndEntry> entries = feed.getEntries();

      // Calcular la fecha límite (hace 8 meses)
      LocalDate currentDate = LocalDate.now();
      LocalDate thresholdDate = currentDate.minusMonths(8);

      int count = 0;
      for (SyndEntry entry : entries) {
        if (count >= 10) break;

        Date pubDate = entry.getPublishedDate();
        if (pubDate == null) continue;

        LocalDate entryDate = pubDate.toInstant().atZone(ZoneId.systemDefault()).toLocalDate();
        if (entryDate.isBefore(thresholdDate)) {
          logger.info("Noticia descartada por antigüedad: {}", entry.getTitle());
          continue;
        }

        String title = entry.getTitle();
        String link = entry.getLink();
        String description = (entry.getDescription() != null) ? entry.getDescription().getValue() : "Sin descripción";
        String finalUrl = resolveFinalUrl(link);

        logger.info("Noticia {}: {}", (count + 1), title);
        logger.info("Enlace: {}", finalUrl);

        newsList.add(new NewsArticle(title, description, finalUrl));
        count++;
      }

      logger.info("Total de noticias procesadas: {}", newsList.size());
    } catch (Exception e) {
      logger.error("Error al obtener las noticias", e);
    }
    return newsList;
  }

  private String resolveFinalUrl(String url) {
    try {
      HttpURLConnection conn = (HttpURLConnection) new URL(url).openConnection();
      conn.setInstanceFollowRedirects(true);
      conn.setRequestProperty("User-Agent", USER_AGENT);
      conn.connect();
      String resolvedUrl = (conn.getHeaderField("Location") != null) ? conn.getHeaderField("Location") : url;

      logger.debug("Redirección resuelta: {} → {}", url, resolvedUrl);
      return resolvedUrl;
    } catch (Exception e) {
      logger.warn("No se pudo resolver la URL: {}", url, e);
      return url;
    }
  }
}

